@@ -13,14 +13,12 @@ init <- function() {
 
     initRprofile()
 
-    setupEnv()
+    setupEnv(init=TRUE)
 
     if (!interactive()) {
       success("Run 'jetpack add <package>' to add packages!")
     } else {
       success("Run 'jetpack::add(package)' to add packages!")
-      enablePackrat()
-      loadNamespace("jetpack", lib.loc=getDefaultLibPaths())
     }
     invisible()
   })
